// generated by go run gen.go; DO NOT EDIT

package icons

var list = []struct {
	name string
	data []byte
}{
	{"Action3DRotation", Action3DRotation},
	{"ActionAccessibility", ActionAccessibility},
	{"ActionAccessible", ActionAccessible},
	{"ActionAccountBalance", ActionAccountBalance},
	{"ActionAccountBalanceWallet", ActionAccountBalanceWallet},
	{"ActionAccountBox", ActionAccountBox},
	{"ActionAccountCircle", ActionAccountCircle},
	{"ActionAddShoppingCart", ActionAddShoppingCart},
	{"ActionAlarm", ActionAlarm},
	{"ActionAlarmAdd", ActionAlarmAdd},
	{"ActionAlarmOff", ActionAlarmOff},
	{"ActionAlarmOn", ActionAlarmOn},
	{"ActionAllOut", ActionAllOut},
	{"ActionAndroid", ActionAndroid},
	{"ActionAnnouncement", ActionAnnouncement},
	{"ActionAspectRatio", ActionAspectRatio},
	{"ActionAssessment", ActionAssessment},
	{"ActionAssignment", ActionAssignment},
	{"ActionAssignmentInd", ActionAssignmentInd},
	{"ActionAssignmentLate", ActionAssignmentLate},
	{"ActionAssignmentReturn", ActionAssignmentReturn},
	{"ActionAssignmentReturned", ActionAssignmentReturned},
	{"ActionAssignmentTurnedIn", ActionAssignmentTurnedIn},
	{"ActionAutorenew", ActionAutorenew},
	{"ActionBackup", ActionBackup},
	{"ActionBook", ActionBook},
	{"ActionBookmark", ActionBookmark},
	{"ActionBookmarkBorder", ActionBookmarkBorder},
	{"ActionBugReport", ActionBugReport},
	{"ActionBuild", ActionBuild},
	{"ActionCached", ActionCached},
	{"ActionCameraEnhance", ActionCameraEnhance},
	{"ActionCardGiftcard", ActionCardGiftcard},
	{"ActionCardMembership", ActionCardMembership},
	{"ActionCardTravel", ActionCardTravel},
	{"ActionChangeHistory", ActionChangeHistory},
	{"ActionCheckCircle", ActionCheckCircle},
	{"ActionChromeReaderMode", ActionChromeReaderMode},
	{"ActionClass", ActionClass},
	{"ActionCode", ActionCode},
	{"ActionCompareArrows", ActionCompareArrows},
	{"ActionCopyright", ActionCopyright},
	{"ActionCreditCard", ActionCreditCard},
	{"ActionDashboard", ActionDashboard},
	{"ActionDateRange", ActionDateRange},
	{"ActionDelete", ActionDelete},
	{"ActionDeleteForever", ActionDeleteForever},
	{"ActionDescription", ActionDescription},
	{"ActionDNS", ActionDNS},
	{"ActionDone", ActionDone},
	{"ActionDoneAll", ActionDoneAll},
	{"ActionDonutLarge", ActionDonutLarge},
	{"ActionDonutSmall", ActionDonutSmall},
	{"ActionEject", ActionEject},
	{"ActionEuroSymbol", ActionEuroSymbol},
	{"ActionEvent", ActionEvent},
	{"ActionEventSeat", ActionEventSeat},
	{"ActionExitToApp", ActionExitToApp},
	{"ActionExplore", ActionExplore},
	{"ActionExtension", ActionExtension},
	{"ActionFace", ActionFace},
	{"ActionFavorite", ActionFavorite},
	{"ActionFavoriteBorder", ActionFavoriteBorder},
	{"ActionFeedback", ActionFeedback},
	{"ActionFindInPage", ActionFindInPage},
	{"ActionFindReplace", ActionFindReplace},
	{"ActionFingerprint", ActionFingerprint},
	{"ActionFlightLand", ActionFlightLand},
	{"ActionFlightTakeoff", ActionFlightTakeoff},
	{"ActionFlipToBack", ActionFlipToBack},
	{"ActionFlipToFront", ActionFlipToFront},
	{"ActionGTranslate", ActionGTranslate},
	{"ActionGavel", ActionGavel},
	{"ActionGetApp", ActionGetApp},
	{"ActionGIF", ActionGIF},
	{"ActionGrade", ActionGrade},
	{"ActionGroupWork", ActionGroupWork},
	{"ActionHelp", ActionHelp},
	{"ActionHelpOutline", ActionHelpOutline},
	{"ActionHighlightOff", ActionHighlightOff},
	{"ActionHistory", ActionHistory},
	{"ActionHome", ActionHome},
	{"ActionHourglassEmpty", ActionHourglassEmpty},
	{"ActionHourglassFull", ActionHourglassFull},
	{"ActionHTTP", ActionHTTP},
	{"ActionHTTPS", ActionHTTPS},
	{"ActionImportantDevices", ActionImportantDevices},
	{"ActionInfo", ActionInfo},
	{"ActionInfoOutline", ActionInfoOutline},
	{"ActionInput", ActionInput},
	{"ActionInvertColors", ActionInvertColors},
	{"ActionLabel", ActionLabel},
	{"ActionLabelOutline", ActionLabelOutline},
	{"ActionLanguage", ActionLanguage},
	{"ActionLaunch", ActionLaunch},
	{"ActionLightbulbOutline", ActionLightbulbOutline},
	{"ActionLineStyle", ActionLineStyle},
	{"ActionLineWeight", ActionLineWeight},
	{"ActionList", ActionList},
	{"ActionLock", ActionLock},
	{"ActionLockOpen", ActionLockOpen},
	{"ActionLockOutline", ActionLockOutline},
	{"ActionLoyalty", ActionLoyalty},
	{"ActionMarkUnreadMailbox", ActionMarkUnreadMailbox},
	{"ActionMotorcycle", ActionMotorcycle},
	{"ActionNoteAdd", ActionNoteAdd},
	{"ActionOfflinePin", ActionOfflinePin},
	{"ActionOpacity", ActionOpacity},
	{"ActionOpenInBrowser", ActionOpenInBrowser},
	{"ActionOpenInNew", ActionOpenInNew},
	{"ActionOpenWith", ActionOpenWith},
	{"ActionPageview", ActionPageview},
	{"ActionPanTool", ActionPanTool},
	{"ActionPayment", ActionPayment},
	{"ActionPermCameraMic", ActionPermCameraMic},
	{"ActionPermContactCalendar", ActionPermContactCalendar},
	{"ActionPermDataSetting", ActionPermDataSetting},
	{"ActionPermDeviceInformation", ActionPermDeviceInformation},
	{"ActionPermIdentity", ActionPermIdentity},
	{"ActionPermMedia", ActionPermMedia},
	{"ActionPermPhoneMsg", ActionPermPhoneMsg},
	{"ActionPermScanWiFi", ActionPermScanWiFi},
	{"ActionPets", ActionPets},
	{"ActionPictureInPicture", ActionPictureInPicture},
	{"ActionPictureInPictureAlt", ActionPictureInPictureAlt},
	{"ActionPlayForWork", ActionPlayForWork},
	{"ActionPolymer", ActionPolymer},
	{"ActionPowerSettingsNew", ActionPowerSettingsNew},
	{"ActionPregnantWoman", ActionPregnantWoman},
	{"ActionPrint", ActionPrint},
	{"ActionQueryBuilder", ActionQueryBuilder},
	{"ActionQuestionAnswer", ActionQuestionAnswer},
	{"ActionReceipt", ActionReceipt},
	{"ActionRecordVoiceOver", ActionRecordVoiceOver},
	{"ActionRedeem", ActionRedeem},
	{"ActionRemoveShoppingCart", ActionRemoveShoppingCart},
	{"ActionReorder", ActionReorder},
	{"ActionReportProblem", ActionReportProblem},
	{"ActionRestore", ActionRestore},
	{"ActionRestorePage", ActionRestorePage},
	{"ActionRoom", ActionRoom},
	{"ActionRoundedCorner", ActionRoundedCorner},
	{"ActionRowing", ActionRowing},
	{"ActionSchedule", ActionSchedule},
	{"ActionSearch", ActionSearch},
	{"ActionSettings", ActionSettings},
	{"ActionSettingsApplications", ActionSettingsApplications},
	{"ActionSettingsBackupRestore", ActionSettingsBackupRestore},
	{"ActionSettingsBluetooth", ActionSettingsBluetooth},
	{"ActionSettingsBrightness", ActionSettingsBrightness},
	{"ActionSettingsCell", ActionSettingsCell},
	{"ActionSettingsEthernet", ActionSettingsEthernet},
	{"ActionSettingsInputAntenna", ActionSettingsInputAntenna},
	{"ActionSettingsInputComponent", ActionSettingsInputComponent},
	{"ActionSettingsInputComposite", ActionSettingsInputComposite},
	{"ActionSettingsInputHDMI", ActionSettingsInputHDMI},
	{"ActionSettingsInputSVideo", ActionSettingsInputSVideo},
	{"ActionSettingsOverscan", ActionSettingsOverscan},
	{"ActionSettingsPhone", ActionSettingsPhone},
	{"ActionSettingsPower", ActionSettingsPower},
	{"ActionSettingsRemote", ActionSettingsRemote},
	{"ActionSettingsVoice", ActionSettingsVoice},
	{"ActionShop", ActionShop},
	{"ActionShopTwo", ActionShopTwo},
	{"ActionShoppingBasket", ActionShoppingBasket},
	{"ActionShoppingCart", ActionShoppingCart},
	{"ActionSpeakerNotes", ActionSpeakerNotes},
	{"ActionSpeakerNotesOff", ActionSpeakerNotesOff},
	{"ActionSpellcheck", ActionSpellcheck},
	{"ActionStarRate", ActionStarRate},
	{"ActionStars", ActionStars},
	{"ActionStore", ActionStore},
	{"ActionSubject", ActionSubject},
	{"ActionSupervisorAccount", ActionSupervisorAccount},
	{"ActionSwapHoriz", ActionSwapHoriz},
	{"ActionSwapVert", ActionSwapVert},
	{"ActionSwapVerticalCircle", ActionSwapVerticalCircle},
	{"ActionSystemUpdateAlt", ActionSystemUpdateAlt},
	{"ActionTab", ActionTab},
	{"ActionTabUnselected", ActionTabUnselected},
	{"ActionTheaters", ActionTheaters},
	{"ActionThumbDown", ActionThumbDown},
	{"ActionThumbUp", ActionThumbUp},
	{"ActionThumbsUpDown", ActionThumbsUpDown},
	{"ActionTimeline", ActionTimeline},
	{"ActionTOC", ActionTOC},
	{"ActionToday", ActionToday},
	{"ActionToll", ActionToll},
	{"ActionTouchApp", ActionTouchApp},
	{"ActionTrackChanges", ActionTrackChanges},
	{"ActionTranslate", ActionTranslate},
	{"ActionTrendingDown", ActionTrendingDown},
	{"ActionTrendingFlat", ActionTrendingFlat},
	{"ActionTrendingUp", ActionTrendingUp},
	{"ActionTurnedIn", ActionTurnedIn},
	{"ActionTurnedInNot", ActionTurnedInNot},
	{"ActionUpdate", ActionUpdate},
	{"ActionVerifiedUser", ActionVerifiedUser},
	{"ActionViewAgenda", ActionViewAgenda},
	{"ActionViewArray", ActionViewArray},
	{"ActionViewCarousel", ActionViewCarousel},
	{"ActionViewColumn", ActionViewColumn},
	{"ActionViewDay", ActionViewDay},
	{"ActionViewHeadline", ActionViewHeadline},
	{"ActionViewList", ActionViewList},
	{"ActionViewModule", ActionViewModule},
	{"ActionViewQuilt", ActionViewQuilt},
	{"ActionViewStream", ActionViewStream},
	{"ActionViewWeek", ActionViewWeek},
	{"ActionVisibility", ActionVisibility},
	{"ActionVisibilityOff", ActionVisibilityOff},
	{"ActionWatchLater", ActionWatchLater},
	{"ActionWork", ActionWork},
	{"ActionYoutubeSearchedFor", ActionYoutubeSearchedFor},
	{"ActionZoomIn", ActionZoomIn},
	{"ActionZoomOut", ActionZoomOut},
	{"AlertAddAlert", AlertAddAlert},
	{"AlertError", AlertError},
	{"AlertErrorOutline", AlertErrorOutline},
	{"AlertWarning", AlertWarning},
	{"AVAddToQueue", AVAddToQueue},
	{"AVAirplay", AVAirplay},
	{"AVAlbum", AVAlbum},
	{"AVArtTrack", AVArtTrack},
	{"AVAVTimer", AVAVTimer},
	{"AVBrandingWatermark", AVBrandingWatermark},
	{"AVCallToAction", AVCallToAction},
	{"AVClosedCaption", AVClosedCaption},
	{"AVEqualizer", AVEqualizer},
	{"AVExplicit", AVExplicit},
	{"AVFastForward", AVFastForward},
	{"AVFastRewind", AVFastRewind},
	{"AVFeaturedPlayList", AVFeaturedPlayList},
	{"AVFeaturedVideo", AVFeaturedVideo},
	{"AVFiberDVR", AVFiberDVR},
	{"AVFiberManualRecord", AVFiberManualRecord},
	{"AVFiberNew", AVFiberNew},
	{"AVFiberPin", AVFiberPin},
	{"AVFiberSmartRecord", AVFiberSmartRecord},
	{"AVForward10", AVForward10},
	{"AVForward30", AVForward30},
	{"AVForward5", AVForward5},
	{"AVGames", AVGames},
	{"AVHD", AVHD},
	{"AVHearing", AVHearing},
	{"AVHighQuality", AVHighQuality},
	{"AVLibraryAdd", AVLibraryAdd},
	{"AVLibraryBooks", AVLibraryBooks},
	{"AVLibraryMusic", AVLibraryMusic},
	{"AVLoop", AVLoop},
	{"AVMic", AVMic},
	{"AVMicNone", AVMicNone},
	{"AVMicOff", AVMicOff},
	{"AVMovie", AVMovie},
	{"AVMusicVideo", AVMusicVideo},
	{"AVNewReleases", AVNewReleases},
	{"AVNotInterested", AVNotInterested},
	{"AVNote", AVNote},
	{"AVPause", AVPause},
	{"AVPauseCircleFilled", AVPauseCircleFilled},
	{"AVPauseCircleOutline", AVPauseCircleOutline},
	{"AVPlayArrow", AVPlayArrow},
	{"AVPlayCircleFilled", AVPlayCircleFilled},
	{"AVPlayCircleOutline", AVPlayCircleOutline},
	{"AVPlaylistAdd", AVPlaylistAdd},
	{"AVPlaylistAddCheck", AVPlaylistAddCheck},
	{"AVPlaylistPlay", AVPlaylistPlay},
	{"AVQueue", AVQueue},
	{"AVQueueMusic", AVQueueMusic},
	{"AVQueuePlayNext", AVQueuePlayNext},
	{"AVRadio", AVRadio},
	{"AVRecentActors", AVRecentActors},
	{"AVRemoveFromQueue", AVRemoveFromQueue},
	{"AVRepeat", AVRepeat},
	{"AVRepeatOne", AVRepeatOne},
	{"AVReplay", AVReplay},
	{"AVReplay10", AVReplay10},
	{"AVReplay30", AVReplay30},
	{"AVReplay5", AVReplay5},
	{"AVShuffle", AVShuffle},
	{"AVSkipNext", AVSkipNext},
	{"AVSkipPrevious", AVSkipPrevious},
	{"AVSlowMotionVideo", AVSlowMotionVideo},
	{"AVSnooze", AVSnooze},
	{"AVSortByAlpha", AVSortByAlpha},
	{"AVStop", AVStop},
	{"AVSubscriptions", AVSubscriptions},
	{"AVSubtitles", AVSubtitles},
	{"AVSurroundSound", AVSurroundSound},
	{"AVVideoCall", AVVideoCall},
	{"AVVideoLabel", AVVideoLabel},
	{"AVVideoLibrary", AVVideoLibrary},
	{"AVVideocam", AVVideocam},
	{"AVVideocamOff", AVVideocamOff},
	{"AVVolumeDown", AVVolumeDown},
	{"AVVolumeMute", AVVolumeMute},
	{"AVVolumeOff", AVVolumeOff},
	{"AVVolumeUp", AVVolumeUp},
	{"AVWeb", AVWeb},
	{"AVWebAsset", AVWebAsset},
	{"CommunicationBusiness", CommunicationBusiness},
	{"CommunicationCall", CommunicationCall},
	{"CommunicationCallEnd", CommunicationCallEnd},
	{"CommunicationCallMade", CommunicationCallMade},
	{"CommunicationCallMerge", CommunicationCallMerge},
	{"CommunicationCallMissed", CommunicationCallMissed},
	{"CommunicationCallMissedOutgoing", CommunicationCallMissedOutgoing},
	{"CommunicationCallReceived", CommunicationCallReceived},
	{"CommunicationCallSplit", CommunicationCallSplit},
	{"CommunicationChat", CommunicationChat},
	{"CommunicationChatBubble", CommunicationChatBubble},
	{"CommunicationChatBubbleOutline", CommunicationChatBubbleOutline},
	{"CommunicationClearAll", CommunicationClearAll},
	{"CommunicationComment", CommunicationComment},
	{"CommunicationContactMail", CommunicationContactMail},
	{"CommunicationContactPhone", CommunicationContactPhone},
	{"CommunicationContacts", CommunicationContacts},
	{"CommunicationDialerSIP", CommunicationDialerSIP},
	{"CommunicationDialpad", CommunicationDialpad},
	{"CommunicationEmail", CommunicationEmail},
	{"CommunicationForum", CommunicationForum},
	{"CommunicationImportContacts", CommunicationImportContacts},
	{"CommunicationImportExport", CommunicationImportExport},
	{"CommunicationInvertColorsOff", CommunicationInvertColorsOff},
	{"CommunicationLiveHelp", CommunicationLiveHelp},
	{"CommunicationLocationOff", CommunicationLocationOff},
	{"CommunicationLocationOn", CommunicationLocationOn},
	{"CommunicationMailOutline", CommunicationMailOutline},
	{"CommunicationMessage", CommunicationMessage},
	{"CommunicationNoSIM", CommunicationNoSIM},
	{"CommunicationPhone", CommunicationPhone},
	{"CommunicationPhoneLinkErase", CommunicationPhoneLinkErase},
	{"CommunicationPhoneLinkLock", CommunicationPhoneLinkLock},
	{"CommunicationPhoneLinkRing", CommunicationPhoneLinkRing},
	{"CommunicationPhoneLinkSetup", CommunicationPhoneLinkSetup},
	{"CommunicationPortableWiFiOff", CommunicationPortableWiFiOff},
	{"CommunicationPresentToAll", CommunicationPresentToAll},
	{"CommunicationRingVolume", CommunicationRingVolume},
	{"CommunicationRSSFeed", CommunicationRSSFeed},
	{"CommunicationScreenShare", CommunicationScreenShare},
	{"CommunicationSpeakerPhone", CommunicationSpeakerPhone},
	{"CommunicationStayCurrentLandscape", CommunicationStayCurrentLandscape},
	{"CommunicationStayCurrentPortrait", CommunicationStayCurrentPortrait},
	{"CommunicationStayPrimaryLandscape", CommunicationStayPrimaryLandscape},
	{"CommunicationStayPrimaryPortrait", CommunicationStayPrimaryPortrait},
	{"CommunicationStopScreenShare", CommunicationStopScreenShare},
	{"CommunicationSwapCalls", CommunicationSwapCalls},
	{"CommunicationTextSMS", CommunicationTextSMS},
	{"CommunicationVoicemail", CommunicationVoicemail},
	{"CommunicationVPNKey", CommunicationVPNKey},
	{"ContentAdd", ContentAdd},
	{"ContentAddBox", ContentAddBox},
	{"ContentAddCircle", ContentAddCircle},
	{"ContentAddCircleOutline", ContentAddCircleOutline},
	{"ContentArchive", ContentArchive},
	{"ContentBackspace", ContentBackspace},
	{"ContentBlock", ContentBlock},
	{"ContentClear", ContentClear},
	{"ContentContentCopy", ContentContentCopy},
	{"ContentContentCut", ContentContentCut},
	{"ContentContentPaste", ContentContentPaste},
	{"ContentCreate", ContentCreate},
	{"ContentDeleteSweep", ContentDeleteSweep},
	{"ContentDrafts", ContentDrafts},
	{"ContentFilterList", ContentFilterList},
	{"ContentFlag", ContentFlag},
	{"ContentFontDownload", ContentFontDownload},
	{"ContentForward", ContentForward},
	{"ContentGesture", ContentGesture},
	{"ContentInbox", ContentInbox},
	{"ContentLink", ContentLink},
	{"ContentLowPriority", ContentLowPriority},
	{"ContentMail", ContentMail},
	{"ContentMarkUnread", ContentMarkUnread},
	{"ContentMoveToInbox", ContentMoveToInbox},
	{"ContentNextWeek", ContentNextWeek},
	{"ContentRedo", ContentRedo},
	{"ContentRemove", ContentRemove},
	{"ContentRemoveCircle", ContentRemoveCircle},
	{"ContentRemoveCircleOutline", ContentRemoveCircleOutline},
	{"ContentReply", ContentReply},
	{"ContentReplyAll", ContentReplyAll},
	{"ContentReport", ContentReport},
	{"ContentSave", ContentSave},
	{"ContentSelectAll", ContentSelectAll},
	{"ContentSend", ContentSend},
	{"ContentSort", ContentSort},
	{"ContentTextFormat", ContentTextFormat},
	{"ContentUnarchive", ContentUnarchive},
	{"ContentUndo", ContentUndo},
	{"ContentWeekend", ContentWeekend},
	{"DeviceAccessAlarm", DeviceAccessAlarm},
	{"DeviceAccessAlarms", DeviceAccessAlarms},
	{"DeviceAccessTime", DeviceAccessTime},
	{"DeviceAddAlarm", DeviceAddAlarm},
	{"DeviceAirplaneModeActive", DeviceAirplaneModeActive},
	{"DeviceAirplaneModeInactive", DeviceAirplaneModeInactive},
	{"DeviceBattery20", DeviceBattery20},
	{"DeviceBattery30", DeviceBattery30},
	{"DeviceBattery50", DeviceBattery50},
	{"DeviceBattery60", DeviceBattery60},
	{"DeviceBattery80", DeviceBattery80},
	{"DeviceBattery90", DeviceBattery90},
	{"DeviceBatteryAlert", DeviceBatteryAlert},
	{"DeviceBatteryCharging20", DeviceBatteryCharging20},
	{"DeviceBatteryCharging30", DeviceBatteryCharging30},
	{"DeviceBatteryCharging50", DeviceBatteryCharging50},
	{"DeviceBatteryCharging60", DeviceBatteryCharging60},
	{"DeviceBatteryCharging80", DeviceBatteryCharging80},
	{"DeviceBatteryCharging90", DeviceBatteryCharging90},
	{"DeviceBatteryChargingFull", DeviceBatteryChargingFull},
	{"DeviceBatteryFull", DeviceBatteryFull},
	{"DeviceBatteryStd", DeviceBatteryStd},
	{"DeviceBatteryUnknown", DeviceBatteryUnknown},
	{"DeviceBluetooth", DeviceBluetooth},
	{"DeviceBluetoothConnected", DeviceBluetoothConnected},
	{"DeviceBluetoothDisabled", DeviceBluetoothDisabled},
	{"DeviceBluetoothSearching", DeviceBluetoothSearching},
	{"DeviceBrightnessAuto", DeviceBrightnessAuto},
	{"DeviceBrightnessHigh", DeviceBrightnessHigh},
	{"DeviceBrightnessLow", DeviceBrightnessLow},
	{"DeviceBrightnessMedium", DeviceBrightnessMedium},
	{"DeviceDataUsage", DeviceDataUsage},
	{"DeviceDeveloperMode", DeviceDeveloperMode},
	{"DeviceDevices", DeviceDevices},
	{"DeviceDVR", DeviceDVR},
	{"DeviceGPSFixed", DeviceGPSFixed},
	{"DeviceGPSNotFixed", DeviceGPSNotFixed},
	{"DeviceGPSOff", DeviceGPSOff},
	{"DeviceGraphicEq", DeviceGraphicEq},
	{"DeviceLocationDisabled", DeviceLocationDisabled},
	{"DeviceLocationSearching", DeviceLocationSearching},
	{"DeviceNetworkCell", DeviceNetworkCell},
	{"DeviceNetworkWiFi", DeviceNetworkWiFi},
	{"DeviceNFC", DeviceNFC},
	{"DeviceScreenLockLandscape", DeviceScreenLockLandscape},
	{"DeviceScreenLockPortrait", DeviceScreenLockPortrait},
	{"DeviceScreenLockRotation", DeviceScreenLockRotation},
	{"DeviceScreenRotation", DeviceScreenRotation},
	{"DeviceSDStorage", DeviceSDStorage},
	{"DeviceSettingsSystemDaydream", DeviceSettingsSystemDaydream},
	{"DeviceSignalCellular0Bar", DeviceSignalCellular0Bar},
	{"DeviceSignalCellular1Bar", DeviceSignalCellular1Bar},
	{"DeviceSignalCellular2Bar", DeviceSignalCellular2Bar},
	{"DeviceSignalCellular3Bar", DeviceSignalCellular3Bar},
	{"DeviceSignalCellular4Bar", DeviceSignalCellular4Bar},
	{"DeviceSignalCellularConnectedNoInternet0Bar", DeviceSignalCellularConnectedNoInternet0Bar},
	{"DeviceSignalCellularConnectedNoInternet1Bar", DeviceSignalCellularConnectedNoInternet1Bar},
	{"DeviceSignalCellularConnectedNoInternet2Bar", DeviceSignalCellularConnectedNoInternet2Bar},
	{"DeviceSignalCellularConnectedNoInternet3Bar", DeviceSignalCellularConnectedNoInternet3Bar},
	{"DeviceSignalCellularConnectedNoInternet4Bar", DeviceSignalCellularConnectedNoInternet4Bar},
	{"DeviceSignalCellularNoSIM", DeviceSignalCellularNoSIM},
	{"DeviceSignalCellularNull", DeviceSignalCellularNull},
	{"DeviceSignalCellularOff", DeviceSignalCellularOff},
	{"DeviceSignalWiFi0Bar", DeviceSignalWiFi0Bar},
	{"DeviceSignalWiFi1Bar", DeviceSignalWiFi1Bar},
	{"DeviceSignalWiFi1BarLock", DeviceSignalWiFi1BarLock},
	{"DeviceSignalWiFi2Bar", DeviceSignalWiFi2Bar},
	{"DeviceSignalWiFi2BarLock", DeviceSignalWiFi2BarLock},
	{"DeviceSignalWiFi3Bar", DeviceSignalWiFi3Bar},
	{"DeviceSignalWiFi3BarLock", DeviceSignalWiFi3BarLock},
	{"DeviceSignalWiFi4Bar", DeviceSignalWiFi4Bar},
	{"DeviceSignalWiFi4BarLock", DeviceSignalWiFi4BarLock},
	{"DeviceSignalWiFiOff", DeviceSignalWiFiOff},
	{"DeviceStorage", DeviceStorage},
	{"DeviceUSB", DeviceUSB},
	{"DeviceWallpaper", DeviceWallpaper},
	{"DeviceWidgets", DeviceWidgets},
	{"DeviceWiFiLock", DeviceWiFiLock},
	{"DeviceWiFiTethering", DeviceWiFiTethering},
	{"EditorAttachFile", EditorAttachFile},
	{"EditorAttachMoney", EditorAttachMoney},
	{"EditorBorderAll", EditorBorderAll},
	{"EditorBorderBottom", EditorBorderBottom},
	{"EditorBorderClear", EditorBorderClear},
	{"EditorBorderColor", EditorBorderColor},
	{"EditorBorderHorizontal", EditorBorderHorizontal},
	{"EditorBorderInner", EditorBorderInner},
	{"EditorBorderLeft", EditorBorderLeft},
	{"EditorBorderOuter", EditorBorderOuter},
	{"EditorBorderRight", EditorBorderRight},
	{"EditorBorderStyle", EditorBorderStyle},
	{"EditorBorderTop", EditorBorderTop},
	{"EditorBorderVertical", EditorBorderVertical},
	{"EditorBubbleChart", EditorBubbleChart},
	{"EditorDragHandle", EditorDragHandle},
	{"EditorFormatAlignCenter", EditorFormatAlignCenter},
	{"EditorFormatAlignJustify", EditorFormatAlignJustify},
	{"EditorFormatAlignLeft", EditorFormatAlignLeft},
	{"EditorFormatAlignRight", EditorFormatAlignRight},
	{"EditorFormatBold", EditorFormatBold},
	{"EditorFormatClear", EditorFormatClear},
	{"EditorFormatColorFill", EditorFormatColorFill},
	{"EditorFormatColorReset", EditorFormatColorReset},
	{"EditorFormatColorText", EditorFormatColorText},
	{"EditorFormatIndentDecrease", EditorFormatIndentDecrease},
	{"EditorFormatIndentIncrease", EditorFormatIndentIncrease},
	{"EditorFormatItalic", EditorFormatItalic},
	{"EditorFormatLineSpacing", EditorFormatLineSpacing},
	{"EditorFormatListBulleted", EditorFormatListBulleted},
	{"EditorFormatListNumbered", EditorFormatListNumbered},
	{"EditorFormatPaint", EditorFormatPaint},
	{"EditorFormatQuote", EditorFormatQuote},
	{"EditorFormatShapes", EditorFormatShapes},
	{"EditorFormatSize", EditorFormatSize},
	{"EditorFormatStrikethrough", EditorFormatStrikethrough},
	{"EditorFormatTextDirectionLToR", EditorFormatTextDirectionLToR},
	{"EditorFormatTextDirectionRToL", EditorFormatTextDirectionRToL},
	{"EditorFormatUnderlined", EditorFormatUnderlined},
	{"EditorFunctions", EditorFunctions},
	{"EditorHighlight", EditorHighlight},
	{"EditorInsertChart", EditorInsertChart},
	{"EditorInsertComment", EditorInsertComment},
	{"EditorInsertDriveFile", EditorInsertDriveFile},
	{"EditorInsertEmoticon", EditorInsertEmoticon},
	{"EditorInsertInvitation", EditorInsertInvitation},
	{"EditorInsertLink", EditorInsertLink},
	{"EditorInsertPhoto", EditorInsertPhoto},
	{"EditorLinearScale", EditorLinearScale},
	{"EditorMergeType", EditorMergeType},
	{"EditorModeComment", EditorModeComment},
	{"EditorModeEdit", EditorModeEdit},
	{"EditorMonetizationOn", EditorMonetizationOn},
	{"EditorMoneyOff", EditorMoneyOff},
	{"EditorMultilineChart", EditorMultilineChart},
	{"EditorPieChart", EditorPieChart},
	{"EditorPieChartOutlined", EditorPieChartOutlined},
	{"EditorPublish", EditorPublish},
	{"EditorShortText", EditorShortText},
	{"EditorShowChart", EditorShowChart},
	{"EditorSpaceBar", EditorSpaceBar},
	{"EditorStrikethroughS", EditorStrikethroughS},
	{"EditorTextFields", EditorTextFields},
	{"EditorTitle", EditorTitle},
	{"EditorVerticalAlignBottom", EditorVerticalAlignBottom},
	{"EditorVerticalAlignCenter", EditorVerticalAlignCenter},
	{"EditorVerticalAlignTop", EditorVerticalAlignTop},
	{"EditorWrapText", EditorWrapText},
	{"FileAttachment", FileAttachment},
	{"FileCloud", FileCloud},
	{"FileCloudCircle", FileCloudCircle},
	{"FileCloudDone", FileCloudDone},
	{"FileCloudDownload", FileCloudDownload},
	{"FileCloudOff", FileCloudOff},
	{"FileCloudQueue", FileCloudQueue},
	{"FileCloudUpload", FileCloudUpload},
	{"FileCreateNewFolder", FileCreateNewFolder},
	{"FileFileDownload", FileFileDownload},
	{"FileFileUpload", FileFileUpload},
	{"FileFolder", FileFolder},
	{"FileFolderOpen", FileFolderOpen},
	{"FileFolderShared", FileFolderShared},
	{"HardwareCast", HardwareCast},
	{"HardwareCastConnected", HardwareCastConnected},
	{"HardwareComputer", HardwareComputer},
	{"HardwareDesktopMac", HardwareDesktopMac},
	{"HardwareDesktopWindows", HardwareDesktopWindows},
	{"HardwareDeveloperBoard", HardwareDeveloperBoard},
	{"HardwareDeviceHub", HardwareDeviceHub},
	{"HardwareDevicesOther", HardwareDevicesOther},
	{"HardwareDock", HardwareDock},
	{"HardwareGamepad", HardwareGamepad},
	{"HardwareHeadset", HardwareHeadset},
	{"HardwareHeadsetMic", HardwareHeadsetMic},
	{"HardwareKeyboard", HardwareKeyboard},
	{"HardwareKeyboardArrowDown", HardwareKeyboardArrowDown},
	{"HardwareKeyboardArrowLeft", HardwareKeyboardArrowLeft},
	{"HardwareKeyboardArrowRight", HardwareKeyboardArrowRight},
	{"HardwareKeyboardArrowUp", HardwareKeyboardArrowUp},
	{"HardwareKeyboardBackspace", HardwareKeyboardBackspace},
	{"HardwareKeyboardCapslock", HardwareKeyboardCapslock},
	{"HardwareKeyboardHide", HardwareKeyboardHide},
	{"HardwareKeyboardReturn", HardwareKeyboardReturn},
	{"HardwareKeyboardTab", HardwareKeyboardTab},
	{"HardwareKeyboardVoice", HardwareKeyboardVoice},
	{"HardwareLaptop", HardwareLaptop},
	{"HardwareLaptopChromebook", HardwareLaptopChromebook},
	{"HardwareLaptopMac", HardwareLaptopMac},
	{"HardwareLaptopWindows", HardwareLaptopWindows},
	{"HardwareMemory", HardwareMemory},
	{"HardwareMouse", HardwareMouse},
	{"HardwarePhoneAndroid", HardwarePhoneAndroid},
	{"HardwarePhoneIPhone", HardwarePhoneIPhone},
	{"HardwarePhoneLink", HardwarePhoneLink},
	{"HardwarePhoneLinkOff", HardwarePhoneLinkOff},
	{"HardwarePowerInput", HardwarePowerInput},
	{"HardwareRouter", HardwareRouter},
	{"HardwareScanner", HardwareScanner},
	{"HardwareSecurity", HardwareSecurity},
	{"HardwareSIMCard", HardwareSIMCard},
	{"HardwareSmartphone", HardwareSmartphone},
	{"HardwareSpeaker", HardwareSpeaker},
	{"HardwareSpeakerGroup", HardwareSpeakerGroup},
	{"HardwareTablet", HardwareTablet},
	{"HardwareTabletAndroid", HardwareTabletAndroid},
	{"HardwareTabletMac", HardwareTabletMac},
	{"HardwareToys", HardwareToys},
	{"HardwareTV", HardwareTV},
	{"HardwareVideogameAsset", HardwareVideogameAsset},
	{"HardwareWatch", HardwareWatch},
	{"ImageAddAPhoto", ImageAddAPhoto},
	{"ImageAddToPhotos", ImageAddToPhotos},
	{"ImageAdjust", ImageAdjust},
	{"ImageAssistant", ImageAssistant},
	{"ImageAssistantPhoto", ImageAssistantPhoto},
	{"ImageAudiotrack", ImageAudiotrack},
	{"ImageBlurCircular", ImageBlurCircular},
	{"ImageBlurLinear", ImageBlurLinear},
	{"ImageBlurOff", ImageBlurOff},
	{"ImageBlurOn", ImageBlurOn},
	{"ImageBrightness1", ImageBrightness1},
	{"ImageBrightness2", ImageBrightness2},
	{"ImageBrightness3", ImageBrightness3},
	{"ImageBrightness4", ImageBrightness4},
	{"ImageBrightness5", ImageBrightness5},
	{"ImageBrightness6", ImageBrightness6},
	{"ImageBrightness7", ImageBrightness7},
	{"ImageBrokenImage", ImageBrokenImage},
	{"ImageBrush", ImageBrush},
	{"ImageBurstMode", ImageBurstMode},
	{"ImageCamera", ImageCamera},
	{"ImageCameraAlt", ImageCameraAlt},
	{"ImageCameraFront", ImageCameraFront},
	{"ImageCameraRear", ImageCameraRear},
	{"ImageCameraRoll", ImageCameraRoll},
	{"ImageCenterFocusStrong", ImageCenterFocusStrong},
	{"ImageCenterFocusWeak", ImageCenterFocusWeak},
	{"ImageCollections", ImageCollections},
	{"ImageCollectionsBookmark", ImageCollectionsBookmark},
	{"ImageColorLens", ImageColorLens},
	{"ImageColorize", ImageColorize},
	{"ImageCompare", ImageCompare},
	{"ImageControlPoint", ImageControlPoint},
	{"ImageControlPointDuplicate", ImageControlPointDuplicate},
	{"ImageCrop", ImageCrop},
	{"ImageCrop169", ImageCrop169},
	{"ImageCrop32", ImageCrop32},
	{"ImageCrop54", ImageCrop54},
	{"ImageCrop75", ImageCrop75},
	{"ImageCropDIN", ImageCropDIN},
	{"ImageCropFree", ImageCropFree},
	{"ImageCropLandscape", ImageCropLandscape},
	{"ImageCropOriginal", ImageCropOriginal},
	{"ImageCropPortrait", ImageCropPortrait},
	{"ImageCropRotate", ImageCropRotate},
	{"ImageCropSquare", ImageCropSquare},
	{"ImageDehaze", ImageDehaze},
	{"ImageDetails", ImageDetails},
	{"ImageEdit", ImageEdit},
	{"ImageExposure", ImageExposure},
	{"ImageExposureNeg1", ImageExposureNeg1},
	{"ImageExposureNeg2", ImageExposureNeg2},
	{"ImageExposurePlus1", ImageExposurePlus1},
	{"ImageExposurePlus2", ImageExposurePlus2},
	{"ImageExposureZero", ImageExposureZero},
	{"ImageFilter", ImageFilter},
	{"ImageFilter1", ImageFilter1},
	{"ImageFilter2", ImageFilter2},
	{"ImageFilter3", ImageFilter3},
	{"ImageFilter4", ImageFilter4},
	{"ImageFilter5", ImageFilter5},
	{"ImageFilter6", ImageFilter6},
	{"ImageFilter7", ImageFilter7},
	{"ImageFilter8", ImageFilter8},
	{"ImageFilter9", ImageFilter9},
	{"ImageFilter9Plus", ImageFilter9Plus},
	{"ImageFilterBAndW", ImageFilterBAndW},
	{"ImageFilterCenterFocus", ImageFilterCenterFocus},
	{"ImageFilterDrama", ImageFilterDrama},
	{"ImageFilterFrames", ImageFilterFrames},
	{"ImageFilterHDR", ImageFilterHDR},
	{"ImageFilterNone", ImageFilterNone},
	{"ImageFilterTiltShift", ImageFilterTiltShift},
	{"ImageFilterVintage", ImageFilterVintage},
	{"ImageFlare", ImageFlare},
	{"ImageFlashAuto", ImageFlashAuto},
	{"ImageFlashOff", ImageFlashOff},
	{"ImageFlashOn", ImageFlashOn},
	{"ImageFlip", ImageFlip},
	{"ImageGradient", ImageGradient},
	{"ImageGrain", ImageGrain},
	{"ImageGridOff", ImageGridOff},
	{"ImageGridOn", ImageGridOn},
	{"ImageHDROff", ImageHDROff},
	{"ImageHDROn", ImageHDROn},
	{"ImageHDRStrong", ImageHDRStrong},
	{"ImageHDRWeak", ImageHDRWeak},
	{"ImageHealing", ImageHealing},
	{"ImageImage", ImageImage},
	{"ImageImageAspectRatio", ImageImageAspectRatio},
	{"ImageISO", ImageISO},
	{"ImageLandscape", ImageLandscape},
	{"ImageLeakAdd", ImageLeakAdd},
	{"ImageLeakRemove", ImageLeakRemove},
	{"ImageLens", ImageLens},
	{"ImageLinkedCamera", ImageLinkedCamera},
	{"ImageLooks", ImageLooks},
	{"ImageLooks3", ImageLooks3},
	{"ImageLooks4", ImageLooks4},
	{"ImageLooks5", ImageLooks5},
	{"ImageLooks6", ImageLooks6},
	{"ImageLooksOne", ImageLooksOne},
	{"ImageLooksTwo", ImageLooksTwo},
	{"ImageLoupe", ImageLoupe},
	{"ImageMonochromePhotos", ImageMonochromePhotos},
	{"ImageMovieCreation", ImageMovieCreation},
	{"ImageMovieFilter", ImageMovieFilter},
	{"ImageMusicNote", ImageMusicNote},
	{"ImageNature", ImageNature},
	{"ImageNaturePeople", ImageNaturePeople},
	{"ImageNavigateBefore", ImageNavigateBefore},
	{"ImageNavigateNext", ImageNavigateNext},
	{"ImagePalette", ImagePalette},
	{"ImagePanorama", ImagePanorama},
	{"ImagePanoramaFishEye", ImagePanoramaFishEye},
	{"ImagePanoramaHorizontal", ImagePanoramaHorizontal},
	{"ImagePanoramaVertical", ImagePanoramaVertical},
	{"ImagePanoramaWideAngle", ImagePanoramaWideAngle},
	{"ImagePhoto", ImagePhoto},
	{"ImagePhotoAlbum", ImagePhotoAlbum},
	{"ImagePhotoCamera", ImagePhotoCamera},
	{"ImagePhotoFilter", ImagePhotoFilter},
	{"ImagePhotoLibrary", ImagePhotoLibrary},
	{"ImagePhotoSizeSelectActual", ImagePhotoSizeSelectActual},
	{"ImagePhotoSizeSelectLarge", ImagePhotoSizeSelectLarge},
	{"ImagePhotoSizeSelectSmall", ImagePhotoSizeSelectSmall},
	{"ImagePictureAsPDF", ImagePictureAsPDF},
	{"ImagePortrait", ImagePortrait},
	{"ImageRemoveRedEye", ImageRemoveRedEye},
	{"ImageRotate90DegreesCCW", ImageRotate90DegreesCCW},
	{"ImageRotateLeft", ImageRotateLeft},
	{"ImageRotateRight", ImageRotateRight},
	{"ImageSlideshow", ImageSlideshow},
	{"ImageStraighten", ImageStraighten},
	{"ImageStyle", ImageStyle},
	{"ImageSwitchCamera", ImageSwitchCamera},
	{"ImageSwitchVideo", ImageSwitchVideo},
	{"ImageTagFaces", ImageTagFaces},
	{"ImageTexture", ImageTexture},
	{"ImageTimeLapse", ImageTimeLapse},
	{"ImageTimer", ImageTimer},
	{"ImageTimer10", ImageTimer10},
	{"ImageTimer3", ImageTimer3},
	{"ImageTimerOff", ImageTimerOff},
	{"ImageTonality", ImageTonality},
	{"ImageTransform", ImageTransform},
	{"ImageTune", ImageTune},
	{"ImageViewComfy", ImageViewComfy},
	{"ImageViewCompact", ImageViewCompact},
	{"ImageVignette", ImageVignette},
	{"ImageWBAuto", ImageWBAuto},
	{"ImageWBCloudy", ImageWBCloudy},
	{"ImageWBIncandescent", ImageWBIncandescent},
	{"ImageWBIridescent", ImageWBIridescent},
	{"ImageWBSunny", ImageWBSunny},
	{"MapsAddLocation", MapsAddLocation},
	{"MapsBeenhere", MapsBeenhere},
	{"MapsDirections", MapsDirections},
	{"MapsDirectionsBike", MapsDirectionsBike},
	{"MapsDirectionsBoat", MapsDirectionsBoat},
	{"MapsDirectionsBus", MapsDirectionsBus},
	{"MapsDirectionsCar", MapsDirectionsCar},
	{"MapsDirectionsRailway", MapsDirectionsRailway},
	{"MapsDirectionsRun", MapsDirectionsRun},
	{"MapsDirectionsSubway", MapsDirectionsSubway},
	{"MapsDirectionsTransit", MapsDirectionsTransit},
	{"MapsDirectionsWalk", MapsDirectionsWalk},
	{"MapsEditLocation", MapsEditLocation},
	{"MapsEVStation", MapsEVStation},
	{"MapsFlight", MapsFlight},
	{"MapsHotel", MapsHotel},
	{"MapsLayers", MapsLayers},
	{"MapsLayersClear", MapsLayersClear},
	{"MapsLocalActivity", MapsLocalActivity},
	{"MapsLocalAirport", MapsLocalAirport},
	{"MapsLocalATM", MapsLocalATM},
	{"MapsLocalBar", MapsLocalBar},
	{"MapsLocalCafe", MapsLocalCafe},
	{"MapsLocalCarWash", MapsLocalCarWash},
	{"MapsLocalConvenienceStore", MapsLocalConvenienceStore},
	{"MapsLocalDining", MapsLocalDining},
	{"MapsLocalDrink", MapsLocalDrink},
	{"MapsLocalFlorist", MapsLocalFlorist},
	{"MapsLocalGasStation", MapsLocalGasStation},
	{"MapsLocalGroceryStore", MapsLocalGroceryStore},
	{"MapsLocalHospital", MapsLocalHospital},
	{"MapsLocalHotel", MapsLocalHotel},
	{"MapsLocalLaundryService", MapsLocalLaundryService},
	{"MapsLocalLibrary", MapsLocalLibrary},
	{"MapsLocalMall", MapsLocalMall},
	{"MapsLocalMovies", MapsLocalMovies},
	{"MapsLocalOffer", MapsLocalOffer},
	{"MapsLocalParking", MapsLocalParking},
	{"MapsLocalPharmacy", MapsLocalPharmacy},
	{"MapsLocalPhone", MapsLocalPhone},
	{"MapsLocalPizza", MapsLocalPizza},
	{"MapsLocalPlay", MapsLocalPlay},
	{"MapsLocalPostOffice", MapsLocalPostOffice},
	{"MapsLocalPrintshop", MapsLocalPrintshop},
	{"MapsLocalSee", MapsLocalSee},
	{"MapsLocalShipping", MapsLocalShipping},
	{"MapsLocalTaxi", MapsLocalTaxi},
	{"MapsMap", MapsMap},
	{"MapsMyLocation", MapsMyLocation},
	{"MapsNavigation", MapsNavigation},
	{"MapsNearMe", MapsNearMe},
	{"MapsPersonPin", MapsPersonPin},
	{"MapsPersonPinCircle", MapsPersonPinCircle},
	{"MapsPinDrop", MapsPinDrop},
	{"MapsPlace", MapsPlace},
	{"MapsRateReview", MapsRateReview},
	{"MapsRestaurant", MapsRestaurant},
	{"MapsRestaurantMenu", MapsRestaurantMenu},
	{"MapsSatellite", MapsSatellite},
	{"MapsStoreMallDirectory", MapsStoreMallDirectory},
	{"MapsStreetView", MapsStreetView},
	{"MapsSubway", MapsSubway},
	{"MapsTerrain", MapsTerrain},
	{"MapsTraffic", MapsTraffic},
	{"MapsTrain", MapsTrain},
	{"MapsTram", MapsTram},
	{"MapsTransferWithinAStation", MapsTransferWithinAStation},
	{"MapsZoomOutMap", MapsZoomOutMap},
	{"NavigationApps", NavigationApps},
	{"NavigationArrowBack", NavigationArrowBack},
	{"NavigationArrowDownward", NavigationArrowDownward},
	{"NavigationArrowDropDown", NavigationArrowDropDown},
	{"NavigationArrowDropDownCircle", NavigationArrowDropDownCircle},
	{"NavigationArrowDropUp", NavigationArrowDropUp},
	{"NavigationArrowForward", NavigationArrowForward},
	{"NavigationArrowUpward", NavigationArrowUpward},
	{"NavigationCancel", NavigationCancel},
	{"NavigationCheck", NavigationCheck},
	{"NavigationChevronLeft", NavigationChevronLeft},
	{"NavigationChevronRight", NavigationChevronRight},
	{"NavigationClose", NavigationClose},
	{"NavigationExpandLess", NavigationExpandLess},
	{"NavigationExpandMore", NavigationExpandMore},
	{"NavigationFirstPage", NavigationFirstPage},
	{"NavigationFullscreen", NavigationFullscreen},
	{"NavigationFullscreenExit", NavigationFullscreenExit},
	{"NavigationLastPage", NavigationLastPage},
	{"NavigationMenu", NavigationMenu},
	{"NavigationMoreHoriz", NavigationMoreHoriz},
	{"NavigationMoreVert", NavigationMoreVert},
	{"NavigationRefresh", NavigationRefresh},
	{"NavigationSubdirectoryArrowLeft", NavigationSubdirectoryArrowLeft},
	{"NavigationSubdirectoryArrowRight", NavigationSubdirectoryArrowRight},
	{"NavigationUnfoldLess", NavigationUnfoldLess},
	{"NavigationUnfoldMore", NavigationUnfoldMore},
	{"NotificationADB", NotificationADB},
	{"NotificationAirlineSeatFlat", NotificationAirlineSeatFlat},
	{"NotificationAirlineSeatFlatAngled", NotificationAirlineSeatFlatAngled},
	{"NotificationAirlineSeatIndividualSuite", NotificationAirlineSeatIndividualSuite},
	{"NotificationAirlineSeatLegroomExtra", NotificationAirlineSeatLegroomExtra},
	{"NotificationAirlineSeatLegroomNormal", NotificationAirlineSeatLegroomNormal},
	{"NotificationAirlineSeatLegroomReduced", NotificationAirlineSeatLegroomReduced},
	{"NotificationAirlineSeatReclineExtra", NotificationAirlineSeatReclineExtra},
	{"NotificationAirlineSeatReclineNormal", NotificationAirlineSeatReclineNormal},
	{"NotificationBluetoothAudio", NotificationBluetoothAudio},
	{"NotificationConfirmationNumber", NotificationConfirmationNumber},
	{"NotificationDiscFull", NotificationDiscFull},
	{"NotificationDoNotDisturb", NotificationDoNotDisturb},
	{"NotificationDoNotDisturbAlt", NotificationDoNotDisturbAlt},
	{"NotificationDoNotDisturbOff", NotificationDoNotDisturbOff},
	{"NotificationDoNotDisturbOn", NotificationDoNotDisturbOn},
	{"NotificationDriveETA", NotificationDriveETA},
	{"NotificationEnhancedEncryption", NotificationEnhancedEncryption},
	{"NotificationEventAvailable", NotificationEventAvailable},
	{"NotificationEventBusy", NotificationEventBusy},
	{"NotificationEventNote", NotificationEventNote},
	{"NotificationFolderSpecial", NotificationFolderSpecial},
	{"NotificationLiveTV", NotificationLiveTV},
	{"NotificationMMS", NotificationMMS},
	{"NotificationMore", NotificationMore},
	{"NotificationNetworkCheck", NotificationNetworkCheck},
	{"NotificationNetworkLocked", NotificationNetworkLocked},
	{"NotificationNoEncryption", NotificationNoEncryption},
	{"NotificationOnDemandVideo", NotificationOnDemandVideo},
	{"NotificationPersonalVideo", NotificationPersonalVideo},
	{"NotificationPhoneBluetoothSpeaker", NotificationPhoneBluetoothSpeaker},
	{"NotificationPhoneForwarded", NotificationPhoneForwarded},
	{"NotificationPhoneInTalk", NotificationPhoneInTalk},
	{"NotificationPhoneLocked", NotificationPhoneLocked},
	{"NotificationPhoneMissed", NotificationPhoneMissed},
	{"NotificationPhonePaused", NotificationPhonePaused},
	{"NotificationPower", NotificationPower},
	{"NotificationPriorityHigh", NotificationPriorityHigh},
	{"NotificationRVHookup", NotificationRVHookup},
	{"NotificationSDCard", NotificationSDCard},
	{"NotificationSIMCardAlert", NotificationSIMCardAlert},
	{"NotificationSMS", NotificationSMS},
	{"NotificationSMSFailed", NotificationSMSFailed},
	{"NotificationSync", NotificationSync},
	{"NotificationSyncDisabled", NotificationSyncDisabled},
	{"NotificationSyncProblem", NotificationSyncProblem},
	{"NotificationSystemUpdate", NotificationSystemUpdate},
	{"NotificationTapAndPlay", NotificationTapAndPlay},
	{"NotificationTimeToLeave", NotificationTimeToLeave},
	{"NotificationVibration", NotificationVibration},
	{"NotificationVoiceChat", NotificationVoiceChat},
	{"NotificationVPNLock", NotificationVPNLock},
	{"NotificationWC", NotificationWC},
	{"NotificationWiFi", NotificationWiFi},
	{"PlacesACUnit", PlacesACUnit},
	{"PlacesAirportShuttle", PlacesAirportShuttle},
	{"PlacesAllInclusive", PlacesAllInclusive},
	{"PlacesBeachAccess", PlacesBeachAccess},
	{"PlacesBusinessCenter", PlacesBusinessCenter},
	{"PlacesCasino", PlacesCasino},
	{"PlacesChildCare", PlacesChildCare},
	{"PlacesChildFriendly", PlacesChildFriendly},
	{"PlacesFitnessCenter", PlacesFitnessCenter},
	{"PlacesFreeBreakfast", PlacesFreeBreakfast},
	{"PlacesGolfCourse", PlacesGolfCourse},
	{"PlacesHotTub", PlacesHotTub},
	{"PlacesKitchen", PlacesKitchen},
	{"PlacesPool", PlacesPool},
	{"PlacesRoomService", PlacesRoomService},
	{"PlacesRVHookup", PlacesRVHookup},
	{"PlacesSmokeFree", PlacesSmokeFree},
	{"PlacesSmokingRooms", PlacesSmokingRooms},
	{"PlacesSpa", PlacesSpa},
	{"SocialCake", SocialCake},
	{"SocialDomain", SocialDomain},
	{"SocialGroup", SocialGroup},
	{"SocialGroupAdd", SocialGroupAdd},
	{"SocialLocationCity", SocialLocationCity},
	{"SocialMood", SocialMood},
	{"SocialMoodBad", SocialMoodBad},
	{"SocialNotifications", SocialNotifications},
	{"SocialNotificationsActive", SocialNotificationsActive},
	{"SocialNotificationsNone", SocialNotificationsNone},
	{"SocialNotificationsOff", SocialNotificationsOff},
	{"SocialNotificationsPaused", SocialNotificationsPaused},
	{"SocialPages", SocialPages},
	{"SocialPartyMode", SocialPartyMode},
	{"SocialPeople", SocialPeople},
	{"SocialPeopleOutline", SocialPeopleOutline},
	{"SocialPerson", SocialPerson},
	{"SocialPersonAdd", SocialPersonAdd},
	{"SocialPersonOutline", SocialPersonOutline},
	{"SocialPlusOne", SocialPlusOne},
	{"SocialPoll", SocialPoll},
	{"SocialPublic", SocialPublic},
	{"SocialSchool", SocialSchool},
	{"SocialSentimentDissatisfied", SocialSentimentDissatisfied},
	{"SocialSentimentNeutral", SocialSentimentNeutral},
	{"SocialSentimentSatisfied", SocialSentimentSatisfied},
	{"SocialSentimentVeryDissatisfied", SocialSentimentVeryDissatisfied},
	{"SocialSentimentVerySatisfied", SocialSentimentVerySatisfied},
	{"SocialShare", SocialShare},
	{"SocialWhatsHot", SocialWhatsHot},
	{"ToggleCheckBox", ToggleCheckBox},
	{"ToggleCheckBoxOutlineBlank", ToggleCheckBoxOutlineBlank},
	{"ToggleIndeterminateCheckBox", ToggleIndeterminateCheckBox},
	{"ToggleRadioButtonChecked", ToggleRadioButtonChecked},
	{"ToggleRadioButtonUnchecked", ToggleRadioButtonUnchecked},
	{"ToggleStar", ToggleStar},
	{"ToggleStarBorder", ToggleStarBorder},
	{"ToggleStarHalf", ToggleStarHalf},
}
