// Copyright 2016 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

//go:generate go run gen.go

// Package colornames provides named colors as defined in the Material Design
// style guide.
//
// See https://material.google.com/style/color.html
package colornames
